// GetDeviceInfo.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"
#include "malloc.h"
#include <windows.h>
#include <conio.h>

#include "CrevisFnIO.h"
#pragma comment(lib, "CrevisFnIO.lib")


void CHECK_FUNCTION(FNIO_VALUE err)
{
	if(err != FNIO_ERROR_SUCCESS)
		throw err;
}

void DisplayLastError(void)
{
	FNIO_VALUE		err_code, size;
	char			szBuff[256] = {0,};
	size = 256;
	LibGetLastError(&err_code, szBuff, &size);
	printf("Error code : 0x%X\n", err_code);
	printf("%s\n", szBuff);
}

BOOL SystemInformations(FNIO_HANDLE hSys)
{
	try
	{
		char			szBuff[256] = {0,};
		FNIO_VALUE		val, size;
		
		printf("****** System informations ******\n");
		
		//
		// Get vendor name
		//
		size = 256;
		CHECK_FUNCTION( SysGetParam(hSys, SYS_VENDOR_NAME, DATA_VALUE, (FNIO_POINTER)szBuff, &size) );
		printf("Vendor name : %s\n", szBuff);
		
		//
		// Get library version
		// 
		size = 256;
		CHECK_FUNCTION( SysGetParam(hSys, SYS_VERSION, DATA_VALUE, (FNIO_POINTER)szBuff, &size) );
		printf("Library version : %s\n", szBuff);
		
		//
		// Get device connection timeout
		//
		size = sizeof(val);
		CHECK_FUNCTION( SysGetParam(hSys, SYS_CONNECTION_TIMEOUT, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("Device connection timeout : %d(ms)\n\n", val);
	}
	catch(...)
	{
		DisplayLastError();
		return FALSE;
	}
	
	return TRUE;
}

FNIO_HANDLE OpenDevice(FNIO_HANDLE hSys)
{
	FNIO_HANDLE hDev = NULL;
	
	try
	{
		DEVICEINFOMODBUSTCP2	Devinfo;
		// Set the ip address of to open device.
		// This example is assumed that ip address is 192.168.0.201.
		Devinfo.IpAddress[0] = 192;	Devinfo.IpAddress[1] = 168;	
		Devinfo.IpAddress[2] = 0;	Devinfo.IpAddress[3] = 201;
		
		//
		// Open device
		//
		CHECK_FUNCTION( DevOpenDevice(hSys, (FNIO_POINTER)&Devinfo, MODBUS_TCP, &hDev) );		
	}
	catch(...)
	{
		DisplayLastError();
		hDev = NULL;
	}
	
	return hDev;
}


BOOL DeviceInformations(FNIO_HANDLE hDev)
{
	try
	{
		char			szBuff[256] = {0,};
		FNIO_VALUE		val, size;
		printf("****** Device informations ******\n");
		
		//
		// Get product code
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_PRODUCT_CODE, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("product code : 0x%X\n", val);
		
		//
		// Get firmware version
		// 
		size = 256;
		CHECK_FUNCTION( DevGetParam(hDev, DEV_FIRMWARE_VERSION, DATA_VALUE, (FNIO_POINTER)szBuff, &size) );
		printf("Firmware version : %s\n", szBuff);
		
		
		//
		// Get product name
		// 
		size = 256;
		CHECK_FUNCTION( DevGetParam(hDev, DEV_PROCDUCT_NAME, DATA_VALUE, (FNIO_POINTER)szBuff, &size) );
		printf("Product name : %s\n", szBuff);
		
		
		//
		// Get firmware release date
		// 
		size = 256;
		CHECK_FUNCTION( DevGetParam(hDev, DEV_FIRMWARE_RELEASE_DATE, DATA_VALUE, (FNIO_POINTER)szBuff, &size) );
		printf("Firmware release date : %s\n", szBuff);
		
		
		//
		// Get IP address
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_IP_ADDRESS, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("IP address : %d.%d.%d.%d\n", ((BYTE*)&val)[0], ((BYTE*)&val)[1], ((BYTE*)&val)[2], ((BYTE*)&val)[3] );
		
		
		//
		// Get subnet address
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_SUBNET_MASK, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("Subnet mask: %d.%d.%d.%d\n", ((BYTE*)&val)[0], ((BYTE*)&val)[1], ((BYTE*)&val)[2], ((BYTE*)&val)[3] );
		
		
		//
		// Get gateway
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_GATEWAY, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("Gateway: %d.%d.%d.%d\n", ((BYTE*)&val)[0], ((BYTE*)&val)[1], ((BYTE*)&val)[2], ((BYTE*)&val)[3] );
		
		
		//
		// Get MAC ID
		// 
		size = 256;
		CHECK_FUNCTION( DevGetParam(hDev, DEV_MAC_ID, DATA_VALUE, (FNIO_POINTER)szBuff, &size) );
		printf("MAC ID : %s\n", szBuff);
		
		
		//
		// Get input image size
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_INPUT_IMAGE_SIZE, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("Input image size : %d\n", val);
		
		
		//
		// Get output image size
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_OUTPUT_IMAGE_SIZE, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("Output image size : %d\n", val);
		
		
		//
		// Get reconnection timeout
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_RECONNECT_TIMEOUT, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("Reconnect timeout : %d(ms)\n", val);
		
		//
		// Get response timeout
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_RESPONSE_TIMEOUT, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("Response timeout : %d(ms)\n", val);
		
		//
		// Get update frequency
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_UPDATE_FREQUENCY, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		printf("IO update frequency : %d(ms)\n", val);
		
		
		//
		// Get auto-connection
		//
		size = sizeof(val);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_AUTO_CONNECTION, DATA_VALUE, (FNIO_POINTER)&val, &size) );
		if(val)
			printf("Auto-connection : TRUE\n\n");
		else
			printf("Auto-connection : FALSE\n\n");
		
	}
	catch(...)
	{
		DisplayLastError();
		return FALSE;
	}
	
	return TRUE;
}


BOOL IoSlotInformations(FNIO_HANDLE hDev)
{
	try
	{
		char			szBuff[256] = {0,};
		FNIO_VALUE		err, val, size, i, slot_num;
		FNIO_HANDLE		hSlot;
		printf("****** IO slot informations ******\n");
		
		size = sizeof(slot_num);
		err = DevGetParam(hDev, DEV_EXPANSION_SLOT_NUMBER, DATA_VALUE, (FNIO_POINTER)&slot_num, &size);
		
		if((err == FNIO_ERROR_NOT_DEFINE_COMMAND) || (slot_num == 0))
		{
			printf("Device has not an expansion slot\n\n");
			return TRUE;
		}
		
		for(i=0; i<slot_num; i++)
		{
			//
			// Get Io module
			//
			CHECK_FUNCTION( IoGetIoModule(hDev, i, &hSlot) );
			
			printf("...%d'th module\n", i);
			//
			// Get module description
			// 
			size = 256;
			CHECK_FUNCTION( IoGetParam(hSlot, IO_MODULE_DESCRIPTION, DATA_VALUE, (FNIO_POINTER)szBuff, &size) );
			printf(".....Module descroption : %s\n", szBuff);
			
			//
			// Get ST number
			//
			size = sizeof(val);
			CHECK_FUNCTION( IoGetParam(hSlot, IO_ST_NUMBER, DATA_VALUE, (FNIO_POINTER)&val, &size) );
			printf(".....ST number : %X\n", val);
			
			
			//
			// Get input bit size
			//
			size = sizeof(val);
			CHECK_FUNCTION( IoGetParam(hSlot, IO_INPUT_BIT_SIZE, DATA_VALUE, (FNIO_POINTER)&val, &size) );
			printf(".....Input bit size : %d\n", val);
			
			//
			// Get output bit size
			//
			size = sizeof(val);
			CHECK_FUNCTION( IoGetParam(hSlot, IO_OUTPUT_BIT_SIZE, DATA_VALUE, (FNIO_POINTER)&val, &size) );
			printf(".....Output bit size : %d\n", val);
			
		}
		printf("\n");
	}
	catch(...)
	{
		DisplayLastError();
		return FALSE;
	}
	
	return TRUE;
}

int main(int argc, char* argv[])
{
	FNIO_HANDLE		hSystem, hDevice;
	FNIO_VALUE		err;

	//
	// Init system
	//
	err = LibInitSystem(&hSystem);
	if(err != FNIO_ERROR_SUCCESS)
	{
		DisplayLastError();
		return 0;
	}
	
	//
	// Display the System informations
	//
	if(!SystemInformations(hSystem))
	{
		LibFreeSystem(hSystem);
		return 0;
	}

	//
	// Open device
	//
	hDevice = OpenDevice(hSystem);
	if(hDevice == NULL)
	{
		LibFreeSystem(hSystem);
		return 0;
	}

	//
	//  Display the device informations
	// 
	if(!DeviceInformations(hDevice))
	{
		DevCloseDevice(hDevice);
		LibFreeSystem(hSystem);
		return 0;
	}

	//
	//  Display the IO slot informations
	// 
	if(!IoSlotInformations(hDevice))
	{
		DevCloseDevice(hDevice);
		LibFreeSystem(hSystem);
		return 0;
	}

	//
	// Close device
	//
	DevCloseDevice(hDevice);

	//
	// Free system
	//
	LibFreeSystem(hSystem);
	return 0;
}

