// IOControl.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "malloc.h"
#include <windows.h>
#include <conio.h>

#include "CrevisFnIO.h"
#pragma comment(lib, "CrevisFnIO.lib")

void CHECK_FUNCTION(FNIO_VALUE err)
{
	if(err != FNIO_ERROR_SUCCESS)
		throw err;
}

void DisplayLastError(void)
{
	FNIO_VALUE		err_code, size;
	char			szBuff[256] = {0,};
	size = 256;
	LibGetLastError(&err_code, szBuff, &size);
	printf("Error code : 0x%X\n", err_code);
	printf("%s\n", szBuff);
}

FNIO_HANDLE OpenDevice(FNIO_HANDLE hSys)
{
	FNIO_HANDLE hDev = NULL;
	
	try
	{
		DEVICEINFOMODBUSTCP2	Devinfo;
		// Set the ip address of to open device.
		// This example is assumed that ip address is 192.168.0.201.
		Devinfo.IpAddress[0] = 192;	Devinfo.IpAddress[1] = 168;	
		Devinfo.IpAddress[2] = 0;	Devinfo.IpAddress[3] = 201;
		
		//
		// Open device
		//
		CHECK_FUNCTION( DevOpenDevice(hSys, (FNIO_POINTER)&Devinfo, MODBUS_TCP, &hDev) );
		
		return hDev;
		
	}
	catch(...)
	{
		DisplayLastError();
		hDev = NULL;
	}
	
	return hDev;
}

void IoDataControls(FNIO_HANDLE hDev)
{
	BYTE*		pinput_image	= NULL;
	BYTE*		poutput_image	= NULL;
	
	try
	{
		FNIO_VALUE	err, val, size, input_size, output_szie, i, output_data;
		
		printf("****** IO data controls ******\n");
		
		//
		// Get input image size
		//
		size = sizeof(input_size);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_INPUT_IMAGE_SIZE, DATA_VALUE, (FNIO_POINTER)&input_size, &size) );
		pinput_image = (BYTE*)malloc(input_size);
		
		//
		// Get output image size
		//
		size = sizeof(output_szie);
		CHECK_FUNCTION( DevGetParam(hDev, DEV_OUTPUT_IMAGE_SIZE, DATA_VALUE, (FNIO_POINTER)&output_szie, &size) );
		poutput_image = (BYTE*)malloc(output_szie);
		
		// ******************************************************
		// Set the environment variable IO data controls
		
		//
		// Set the IO data update frequency to maximum speed.
		//
		val = 0;	//0 ms 
		size = sizeof(val);
		CHECK_FUNCTION( DevSetParam(hDev, DEV_UPDATE_FREQUENCY, &val, &size) );
		
		//
		// Set the response timeout to 1 second. 
		//
		val = 1000;	//0 ms 
		size = sizeof(val);
		CHECK_FUNCTION( DevSetParam(hDev, DEV_RESPONSE_TIMEOUT, &val, &size) );
		// *****************************************************
		
		printf("Press <Enter> to srart and stop.\n\n");
		
		getch();
		
		//
		// IO data update start
		//
		DevIoUpdateStart(hDev);
		
		output_data = 0;
		do
		{
			output_data = (output_data%255);
			
			//
			// Write output image data
			//
			if(output_szie > 0)
			{
				memset(poutput_image, output_data, output_szie);
				
				err = DevWriteOutputImage(hDev, 0, poutput_image, output_szie);
				if(err == FNIO_ERROR_SUCCESS)
				{
					printf("Write output image data : ");
					for(i=0; i<output_szie; i++)
					{
						printf("%02X", poutput_image[i]);
					}
					printf("\n");
					
				}
				else if(err == FNIO_ERROR_NOT_AVAILABLE_PORT)
				{
					printf("Port is not available\n");
				}
				else
				{
					DisplayLastError();
				}
			}
			else
			{
				printf("Devic has not output image\n");
			}
			
			Sleep(100);
			
			//
			// Read input image data
			//
			if(input_size > 0)
			{
				err = DevReadInputImage(hDev, 0, pinput_image, input_size);
				if(err == FNIO_ERROR_SUCCESS)
				{
					printf("Read input image data   : ");
					for(i=0; i<input_size; i++)
					{
						printf("%02X", pinput_image[i]);
					}
					printf("\n");
					
				}
				else if(err == FNIO_ERROR_NOT_AVAILABLE_PORT)
				{
					printf("Port is not available\n");
				}
				else
				{
					DisplayLastError();
				}
			}
			else
			{
				printf("Devic has not an input image\n");
			}
			
			Sleep(100);
			
			output_data++;
			
		}while(!kbhit());
		getch();
		
		//
		// IO data update stop
		//
		DevIoUpdateStop(hDev);
	}
	catch(...)
	{
		DisplayLastError();
	}
	
	if(pinput_image)
		free(pinput_image);
	if(poutput_image)
		free(poutput_image);
}


int main(int argc, char* argv[])
{
	FNIO_HANDLE		hSystem, hDevice;
	FNIO_VALUE		err;
	
	//
	// Init system
	//
	err = LibInitSystem(&hSystem);
	if(err != FNIO_ERROR_SUCCESS)
	{
		DisplayLastError();
		return 0;
	}
	
	//
	// Open device
	//
	hDevice = OpenDevice(hSystem);
	if(hDevice == NULL)
	{
		LibFreeSystem(hSystem);
		return 0;
	}
	
	//
	// IO data control
	//
	IoDataControls(hDevice);

	//
	// Close device
	//
	DevCloseDevice(hDevice);
	
	//
	// Free system
	//
	LibFreeSystem(hSystem);
	return 0;
}
